@@ -19,7 +19,7 @@
 //!
 //!
 //! Example
-//! ```rust
+//! ```rust no_run
 //! use inquire::{min_length, Text};
 //!
 //! fn main() {
@@ -36,14 +36,13 @@
 
 #![warn(missing_docs)]
 
-#[macro_use]
-extern crate simple_error;
-
 mod answer;
 pub mod config;
 #[cfg(feature = "date")]
 mod date_utils;
+pub mod error;
 mod formatter;
+mod key;
 mod parser;
 mod prompts;
 mod renderer;
@@ -53,11 +52,3 @@ pub mod validator;
 
 pub use crate::prompts::*;
 pub use answer::OptionAnswer;
-
-#[cfg(test)]
-mod tests {
-    #[test]
-    fn it_works() {
-        assert_eq!(2 + 2, 4);
-    }
-}
