@@ -1,12 +1,12 @@
-use std::{collections::HashSet, error::Error, iter::FromIterator};
+use std::{collections::HashSet, iter::FromIterator};
 use unicode_segmentation::UnicodeSegmentation;
 
-use termion::event::Key;
-
 use crate::{
     answer::OptionAnswer,
     config::{self, Filter},
+    error::{InquireError, InquireResult},
     formatter::{self, MultiOptionFormatter},
+    key::{Key, KeyModifiers},
     renderer::Renderer,
     terminal::Terminal,
     utils::paginate,
@@ -50,7 +50,7 @@ pub struct MultiSelect<'a> {
     pub formatter: MultiOptionFormatter,
 
     /// Validator to apply to the user input.
-    pub validator: Option<MultiOptionValidator>,
+    pub validator: Option<MultiOptionValidator<'a>>,
 }
 
 impl<'a> MultiSelect<'a> {
@@ -130,7 +130,7 @@ impl<'a> MultiSelect<'a> {
     }
 
     /// Adds a validator to the collection of validators.
-    pub fn with_validator(mut self, validator: MultiOptionValidator) -> Self {
+    pub fn with_validator(mut self, validator: MultiOptionValidator<'a>) -> Self {
         self.validator = Some(validator);
         self
     }
@@ -149,7 +149,7 @@ impl<'a> MultiSelect<'a> {
 
     /// Parses the provided behavioral and rendering options and prompts
     /// the CLI user for input according to them.
-    pub fn prompt(self) -> Result<Vec<OptionAnswer>, Box<dyn Error>> {
+    pub fn prompt(self) -> InquireResult<Vec<OptionAnswer>> {
         let terminal = Terminal::new()?;
         let mut renderer = Renderer::new(terminal)?;
         self.prompt_with_renderer(&mut renderer)
@@ -158,7 +158,7 @@ impl<'a> MultiSelect<'a> {
     pub(in crate) fn prompt_with_renderer(
         self,
         renderer: &mut Renderer,
-    ) -> Result<Vec<OptionAnswer>, Box<dyn Error>> {
+    ) -> InquireResult<Vec<OptionAnswer>> {
         MultiSelectPrompt::new(self)?.prompt(renderer)
     }
 }
@@ -176,19 +176,25 @@ struct MultiSelectPrompt<'a> {
     filtered_options: Vec<usize>,
     filter: Filter,
     formatter: MultiOptionFormatter,
-    validator: Option<MultiOptionValidator>,
+    validator: Option<MultiOptionValidator<'a>>,
     error: Option<String>,
 }
 
 impl<'a> MultiSelectPrompt<'a> {
-    fn new(mso: MultiSelect<'a>) -> Result<Self, Box<dyn Error>> {
+    fn new(mso: MultiSelect<'a>) -> InquireResult<Self> {
         if mso.options.is_empty() {
-            bail!("Please provide options to select from");
+            return Err(InquireError::InvalidConfiguration(
+                "Available options can not be empty".into(),
+            ));
         }
         if let Some(default) = mso.default {
             for i in default {
                 if i >= &mso.options.len() {
-                    bail!("Invalid index, larger than options available");
+                    return Err(InquireError::InvalidConfiguration(format!(
+                        "Index {} is out-of-bounds for length {} of options",
+                        i,
+                        &mso.options.len()
+                    )));
                 }
             }
         }
@@ -261,14 +267,11 @@ impl<'a> MultiSelectPrompt<'a> {
         let old_filter = self.filter_value.clone();
 
         match key {
-            Key::Up => self.move_cursor_up(),
-            Key::Char('k') if self.vim_mode => self.move_cursor_up(),
-            Key::Char('\t') | Key::Down => self.move_cursor_down(),
-            Key::Char('j') if self.vim_mode => self.move_cursor_down(),
-            Key::Char(' ') => self.toggle_cursor_selection(),
-            Key::Char('\x17') | Key::Char('\x18') => {
-                self.filter_value = None;
-            }
+            Key::Up(KeyModifiers::NONE) => self.move_cursor_up(),
+            Key::Char('k', KeyModifiers::NONE) if self.vim_mode => self.move_cursor_up(),
+            Key::Tab | Key::Down(KeyModifiers::NONE) => self.move_cursor_down(),
+            Key::Char('j', KeyModifiers::NONE) if self.vim_mode => self.move_cursor_down(),
+            Key::Char(' ', KeyModifiers::NONE) => self.toggle_cursor_selection(),
             Key::Backspace => {
                 if let Some(filter) = &self.filter_value {
                     let len = filter[..].graphemes(true).count();
@@ -276,7 +279,7 @@ impl<'a> MultiSelectPrompt<'a> {
                     self.filter_value = Some(filter[..].graphemes(true).take(new_len).collect());
                 }
             }
-            Key::Right => {
+            Key::Right(KeyModifiers::NONE) => {
                 self.checked.clear();
                 for idx in &self.filtered_options {
                     self.checked.insert(*idx);
@@ -286,14 +289,14 @@ impl<'a> MultiSelectPrompt<'a> {
                     self.filter_value = None;
                 }
             }
-            Key::Left => {
+            Key::Left(KeyModifiers::NONE) => {
                 self.checked.clear();
 
                 if !self.keep_filter {
                     self.filter_value = None;
                 }
             }
-            Key::Char(c) => match &mut self.filter_value {
+            Key::Char(c, KeyModifiers::NONE) => match &mut self.filter_value {
                 Some(val) => val.push(c),
                 None => self.filter_value = Some(String::from(c)),
             },
@@ -330,7 +333,7 @@ impl<'a> MultiSelectPrompt<'a> {
         return Ok(selected_options);
     }
 
-    fn render(&mut self, renderer: &mut Renderer) -> Result<(), std::io::Error> {
+    fn render(&mut self, renderer: &mut Renderer) -> InquireResult<()> {
         let prompt = &self.message;
 
         renderer.reset_prompt()?;
@@ -367,7 +370,7 @@ impl<'a> MultiSelectPrompt<'a> {
         Ok(())
     }
 
-    fn prompt(mut self, renderer: &mut Renderer) -> Result<Vec<OptionAnswer>, Box<dyn Error>> {
+    fn prompt(mut self, renderer: &mut Renderer) -> InquireResult<Vec<OptionAnswer>> {
         let final_answer: Vec<OptionAnswer>;
 
         loop {
@@ -376,8 +379,8 @@ impl<'a> MultiSelectPrompt<'a> {
             let key = renderer.read_key()?;
 
             match key {
-                Key::Ctrl('c') => bail!("Multi-selection interrupted by ctrl-c"),
-                Key::Char('\n') | Key::Char('\r') => match self.get_final_answer() {
+                Key::Cancel => return Err(InquireError::OperationCanceled),
+                Key::Submit => match self.get_final_answer() {
                     Ok(answer) => {
                         final_answer = answer;
                         break;
