@@ -1,10 +1,12 @@
-use std::error::Error;
 use unicode_segmentation::UnicodeSegmentation;
 
-use termion::event::Key;
-
 use crate::{
-    formatter::StringFormatter, renderer::Renderer, terminal::Terminal, validator::StringValidator,
+    error::{InquireError, InquireResult},
+    formatter::StringFormatter,
+    key::{Key, KeyModifiers},
+    renderer::Renderer,
+    terminal::Terminal,
+    validator::StringValidator,
 };
 /// Presents a message to the user and retrieves a single line of text input.
 ///
@@ -25,14 +27,14 @@ pub struct Password<'a> {
 
     /// Collection of validators to apply to the user input.
     /// Validation errors are displayed to the user one line above the prompt.
-    pub validators: Vec<StringValidator>,
+    pub validators: Vec<StringValidator<'a>>,
 }
 
 impl<'a> Password<'a> {
     /// Default formatter.
     pub const DEFAULT_FORMATTER: StringFormatter = |_| "********";
     /// Default collection of validators.
-    pub const DEFAULT_VALIDATORS: Vec<StringValidator> = Vec::new();
+    pub const DEFAULT_VALIDATORS: Vec<StringValidator<'a>> = Vec::new();
     /// Default help message.
     pub const DEFAULT_HELP_MESSAGE: Option<&'a str> = None;
 
@@ -59,13 +61,13 @@ impl<'a> Password<'a> {
     }
 
     /// Adds a validator to the collection of validators.
-    pub fn with_validator(mut self, validator: StringValidator) -> Self {
+    pub fn with_validator(mut self, validator: StringValidator<'a>) -> Self {
         self.validators.push(validator);
         self
     }
 
     /// Adds the validators to the collection of validators.
-    pub fn with_validators(mut self, validators: &[StringValidator]) -> Self {
+    pub fn with_validators(mut self, validators: &[StringValidator<'a>]) -> Self {
         for validator in validators {
             self.validators.push(validator.clone());
         }
@@ -74,16 +76,13 @@ impl<'a> Password<'a> {
 
     /// Parses the provided behavioral and rendering options and prompts
     /// the CLI user for input according to them.
-    pub fn prompt(self) -> Result<String, Box<dyn Error>> {
+    pub fn prompt(self) -> InquireResult<String> {
         let terminal = Terminal::new()?;
         let mut renderer = Renderer::new(terminal)?;
         self.prompt_with_renderer(&mut renderer)
     }
 
-    pub(in crate) fn prompt_with_renderer(
-        self,
-        renderer: &mut Renderer,
-    ) -> Result<String, Box<dyn Error>> {
+    pub(in crate) fn prompt_with_renderer(self, renderer: &mut Renderer) -> InquireResult<String> {
         PasswordPrompt::from(self).prompt(renderer)
     }
 }
@@ -93,7 +92,7 @@ struct PasswordPrompt<'a> {
     help_message: Option<&'a str>,
     content: String,
     formatter: StringFormatter,
-    validators: Vec<StringValidator>,
+    validators: Vec<StringValidator<'a>>,
     error: Option<String>,
 }
 
@@ -124,7 +123,7 @@ impl<'a> PasswordPrompt<'a> {
                 let new_len = len.saturating_sub(1);
                 self.content = self.content[..].graphemes(true).take(new_len).collect();
             }
-            Key::Char(c) => self.content.push(c),
+            Key::Char(c, KeyModifiers::NONE) => self.content.push(c),
             _ => {}
         }
     }
@@ -140,7 +139,7 @@ impl<'a> PasswordPrompt<'a> {
         Ok(self.content.clone())
     }
 
-    fn render(&mut self, renderer: &mut Renderer) -> Result<(), std::io::Error> {
+    fn render(&mut self, renderer: &mut Renderer) -> InquireResult<()> {
         let prompt = &self.message;
 
         renderer.reset_prompt()?;
@@ -160,7 +159,7 @@ impl<'a> PasswordPrompt<'a> {
         Ok(())
     }
 
-    fn prompt(mut self, renderer: &mut Renderer) -> Result<String, Box<dyn Error>> {
+    fn prompt(mut self, renderer: &mut Renderer) -> InquireResult<String> {
         let final_answer: String;
 
         loop {
@@ -169,8 +168,8 @@ impl<'a> PasswordPrompt<'a> {
             let key = renderer.read_key()?;
 
             match key {
-                Key::Ctrl('c') => bail!("Password input interrupted by ctrl-c"),
-                Key::Char('\n') | Key::Char('\r') => match self.get_final_answer() {
+                Key::Cancel => return Err(InquireError::OperationCanceled),
+                Key::Submit => match self.get_final_answer() {
                     Ok(answer) => {
                         final_answer = answer;
                         break;
@@ -189,14 +188,21 @@ impl<'a> PasswordPrompt<'a> {
 
 #[cfg(test)]
 mod test {
+    use super::Password;
+    use crate::{renderer::Renderer, terminal::Terminal};
+    use crossterm::event::{KeyCode, KeyEvent};
     use ntest::timeout;
 
-    use crate::{renderer::Renderer, terminal::Terminal, Password};
-
     fn default<'a>() -> Password<'a> {
         Password::new("Question?")
     }
 
+    macro_rules! text_to_events {
+        ($text:expr) => {{
+            $text.chars().map(|c| KeyCode::Char(c))
+        }};
+    }
+
     macro_rules! password_test {
         ($name:ident,$input:expr,$output:expr) => {
             password_test! {$name, $input, $output, default()}
@@ -206,10 +212,11 @@ mod test {
             #[test]
             #[timeout(100)]
             fn $name() {
-                let mut read: &[u8] = $input.as_bytes();
+                let read: Vec<KeyEvent> = $input.into_iter().map(KeyEvent::from).collect();
+                let mut read = read.iter();
 
                 let mut write: Vec<u8> = Vec::new();
-                let terminal = Terminal::new_with_io(&mut write, &mut read).unwrap();
+                let terminal = Terminal::new_with_io(&mut write, &mut read);
                 let mut renderer = Renderer::new(terminal).unwrap();
 
                 let ans = $prompt.prompt_with_renderer(&mut renderer).unwrap();
@@ -219,35 +226,77 @@ mod test {
         };
     }
 
-    password_test!(empty, "\n", "");
+    password_test!(empty, vec![KeyCode::Enter], "");
 
-    password_test!(single_letter, "b\n", "b");
+    password_test!(single_letter, vec![KeyCode::Char('b'), KeyCode::Enter], "b");
 
-    password_test!(letters_and_enter, "normal input\n", "normal input");
+    password_test!(
+        letters_and_enter,
+        text_to_events!("normal input\n"),
+        "normal input"
+    );
 
     password_test!(
         letters_and_enter_with_emoji,
-        "with emoji 🧘🏻‍♂️, 🌍, 🍞, 🚗, 📞\n",
+        text_to_events!("with emoji 🧘🏻‍♂️, 🌍, 🍞, 🚗, 📞\n"),
         "with emoji 🧘🏻‍♂️, 🌍, 🍞, 🚗, 📞"
     );
 
     password_test!(
         input_and_correction,
-        "anor\x7F\x7F\x7F\x7Fnormal input\n",
+        {
+            let mut events = vec![];
+            events.append(&mut text_to_events!("anor").collect());
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.append(&mut text_to_events!("normal input").collect());
+            events.push(KeyCode::Enter);
+            events
+        },
         "normal input"
     );
 
     password_test!(
         input_and_excessive_correction,
-        "anor\x7F\x7F\x7F\x7F\x7F\x7F\x7F\x7Fnormal input\n",
+        {
+            let mut events = vec![];
+            events.append(&mut text_to_events!("anor").collect());
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.append(&mut text_to_events!("normal input").collect());
+            events.push(KeyCode::Enter);
+            events
+        },
         "normal input"
     );
 
     password_test!(
         input_correction_after_validation,
-        "1234567890\n\x7F\x7F\x7F\x7F\x7F\nyes\n",
+        {
+            let mut events = vec![];
+            events.append(&mut text_to_events!("1234567890").collect());
+            events.push(KeyCode::Enter);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.append(&mut text_to_events!("yes").collect());
+            events.push(KeyCode::Enter);
+            events
+        },
         "12345yes",
-        Password::new("").with_validator(|ans| match ans.len() {
+        Password::new("").with_validator(&|ans| match ans.len() {
             len if len > 5 && len < 10 => Ok(()),
             _ => Err("Invalid".to_string()),
         })
