@@ -1,13 +1,12 @@
-use simple_error::SimpleError;
-use std::{error::Error, iter::FromIterator};
+use std::iter::FromIterator;
 use unicode_segmentation::UnicodeSegmentation;
 
-use termion::event::Key;
-
 use crate::{
     answer::OptionAnswer,
     config::{self, Filter},
+    error::{InquireError, InquireResult},
     formatter::{self, OptionFormatter},
+    key::{Key, KeyModifiers},
     renderer::Renderer,
     terminal::Terminal,
     utils::paginate,
@@ -117,7 +116,7 @@ impl<'a> Select<'a> {
 
     /// Parses the provided behavioral and rendering options and prompts
     /// the CLI user for input according to them.
-    pub fn prompt(self) -> Result<OptionAnswer, Box<dyn Error>> {
+    pub fn prompt(self) -> InquireResult<OptionAnswer> {
         let terminal = Terminal::new()?;
         let mut renderer = Renderer::new(terminal)?;
         self.prompt_with_renderer(&mut renderer)
@@ -126,7 +125,7 @@ impl<'a> Select<'a> {
     pub(in crate) fn prompt_with_renderer(
         self,
         renderer: &mut Renderer,
-    ) -> Result<OptionAnswer, Box<dyn Error>> {
+    ) -> InquireResult<OptionAnswer> {
         SelectPrompt::new(self)?.prompt(renderer)
     }
 }
@@ -142,17 +141,22 @@ struct SelectPrompt<'a> {
     filtered_options: Vec<usize>,
     filter: Filter,
     formatter: OptionFormatter,
-    error: Option<Box<dyn Error>>,
 }
 
 impl<'a> SelectPrompt<'a> {
-    fn new(so: Select<'a>) -> Result<Self, Box<dyn Error>> {
+    fn new(so: Select<'a>) -> InquireResult<Self> {
         if so.options.is_empty() {
-            bail!("Please provide options to select from");
+            return Err(InquireError::InvalidConfiguration(
+                "Available options can not be empty".into(),
+            ));
         }
 
         if so.starting_cursor >= so.options.len() {
-            bail!("Starting selection should not be larger than length of options");
+            return Err(InquireError::InvalidConfiguration(format!(
+                "Starting cursor index {} is out-of-bounds for length {} of options",
+                so.starting_cursor,
+                &so.options.len()
+            )));
         }
 
         Ok(Self {
@@ -166,7 +170,6 @@ impl<'a> SelectPrompt<'a> {
             filtered_options: Vec::from_iter(0..so.options.len()),
             filter: so.filter,
             formatter: so.formatter,
-            error: None,
         })
     }
 
@@ -201,13 +204,10 @@ impl<'a> SelectPrompt<'a> {
         let old_filter = self.filter_value.clone();
 
         match key {
-            Key::Up => self.move_cursor_up(),
-            Key::Char('k') if self.vim_mode => self.move_cursor_up(),
-            Key::Char('\t') | Key::Down => self.move_cursor_down(),
-            Key::Char('j') if self.vim_mode => self.move_cursor_down(),
-            Key::Char('\x17') | Key::Char('\x18') => {
-                self.filter_value = None;
-            }
+            Key::Up(KeyModifiers::NONE) => self.move_cursor_up(),
+            Key::Char('k', KeyModifiers::NONE) if self.vim_mode => self.move_cursor_up(),
+            Key::Tab | Key::Down(KeyModifiers::NONE) => self.move_cursor_down(),
+            Key::Char('j', KeyModifiers::NONE) if self.vim_mode => self.move_cursor_down(),
             Key::Backspace => {
                 if let Some(filter) = &self.filter_value {
                     let len = filter[..].graphemes(true).count();
@@ -215,7 +215,7 @@ impl<'a> SelectPrompt<'a> {
                     self.filter_value = Some(filter[..].graphemes(true).take(new_len).collect());
                 }
             }
-            Key::Char(c) => match &mut self.filter_value {
+            Key::Char(c, KeyModifiers::NONE) => match &mut self.filter_value {
                 Some(val) => val.push(c),
                 None => self.filter_value = Some(String::from(c)),
             },
@@ -231,22 +231,22 @@ impl<'a> SelectPrompt<'a> {
         }
     }
 
-    fn get_final_answer(&self) -> Result<OptionAnswer, Box<dyn Error>> {
+    fn get_final_answer(&self) -> InquireResult<OptionAnswer> {
         self.filtered_options
             .get(self.cursor_index)
             .and_then(|i| self.options.get(*i).map(|opt| OptionAnswer::new(*i, opt)))
-            .ok_or(Box::new(SimpleError::new("Invalid selected index")))
+            .ok_or(InquireError::InvalidState(format!(
+                "Index {} is not in the range of options with len {}",
+                self.cursor_index,
+                self.options.len()
+            )))
     }
 
-    fn render(&mut self, renderer: &mut Renderer) -> Result<(), std::io::Error> {
+    fn render(&mut self, renderer: &mut Renderer) -> InquireResult<()> {
         let prompt = &self.message;
 
         renderer.reset_prompt()?;
 
-        if let Some(err) = &self.error {
-            renderer.print_error(&**err)?;
-        }
-
         renderer.print_prompt(&prompt, None, self.filter_value.as_deref())?;
 
         let choices = self
@@ -271,7 +271,7 @@ impl<'a> SelectPrompt<'a> {
         Ok(())
     }
 
-    fn prompt(mut self, renderer: &mut Renderer) -> Result<OptionAnswer, Box<dyn Error>> {
+    fn prompt(mut self, renderer: &mut Renderer) -> InquireResult<OptionAnswer> {
         let final_answer: OptionAnswer;
 
         loop {
@@ -280,14 +280,13 @@ impl<'a> SelectPrompt<'a> {
             let key = renderer.read_key()?;
 
             match key {
-                Key::Ctrl('c') => bail!("Multi-selection interrupted by ctrl-c"),
-                Key::Char('\n') | Key::Char('\r') | Key::Char(' ') => match self.get_final_answer()
-                {
+                Key::Cancel => return Err(InquireError::OperationCanceled),
+                Key::Submit | Key::Char(' ', KeyModifiers::NONE) => match self.get_final_answer() {
                     Ok(answer) => {
                         final_answer = answer;
                         break;
                     }
-                    Err(err) => self.error = Some(err),
+                    Err(err) => return Err(err),
                 },
                 key => self.on_change(key),
             }
