@@ -1,12 +1,11 @@
-use std::error::Error;
 use unicode_segmentation::UnicodeSegmentation;
 
-use termion::event::Key;
-
 use crate::{
     answer::OptionAnswer,
     config::{self, Suggester},
+    error::{InquireError, InquireResult},
     formatter::{StringFormatter, DEFAULT_STRING_FORMATTER},
+    key::{Key, KeyModifiers},
     renderer::Renderer,
     terminal::Terminal,
     utils::paginate,
@@ -32,7 +31,7 @@ pub struct Text<'a> {
 
     /// Collection of validators to apply to the user input.
     /// Validation errors are displayed to the user one line above the prompt.
-    pub validators: Vec<StringValidator>,
+    pub validators: Vec<StringValidator<'a>>,
 
     /// Page size of the suggestions displayed to the user, when applicable.
     pub page_size: usize,
@@ -47,7 +46,7 @@ impl<'a> Text<'a> {
     /// Default formatter.
     pub const DEFAULT_FORMATTER: StringFormatter = DEFAULT_STRING_FORMATTER;
     /// Default collection of validators.
-    pub const DEFAULT_VALIDATORS: Vec<StringValidator> = Vec::new();
+    pub const DEFAULT_VALIDATORS: Vec<StringValidator<'a>> = Vec::new();
     /// Default help message.
     pub const DEFAULT_HELP_MESSAGE: Option<&'a str> = None;
 
@@ -89,13 +88,13 @@ impl<'a> Text<'a> {
     }
 
     /// Adds a validator to the collection of validators.
-    pub fn with_validator(mut self, validator: StringValidator) -> Self {
+    pub fn with_validator(mut self, validator: StringValidator<'a>) -> Self {
         self.validators.push(validator);
         self
     }
 
     /// Adds the validators to the collection of validators.
-    pub fn with_validators(mut self, validators: &[StringValidator]) -> Self {
+    pub fn with_validators(mut self, validators: &[StringValidator<'a>]) -> Self {
         for validator in validators {
             self.validators.push(validator.clone());
         }
@@ -104,16 +103,13 @@ impl<'a> Text<'a> {
 
     /// Parses the provided behavioral and rendering options and prompts
     /// the CLI user for input according to them.
-    pub fn prompt(self) -> Result<String, Box<dyn Error>> {
+    pub fn prompt(self) -> InquireResult<String> {
         let terminal = Terminal::new()?;
         let mut renderer = Renderer::new(terminal)?;
         self.prompt_with_renderer(&mut renderer)
     }
 
-    pub(in crate) fn prompt_with_renderer(
-        self,
-        renderer: &mut Renderer,
-    ) -> Result<String, Box<dyn Error>> {
+    pub(in crate) fn prompt_with_renderer(self, renderer: &mut Renderer) -> InquireResult<String> {
         TextPrompt::from(self).prompt(renderer)
     }
 }
@@ -122,14 +118,14 @@ impl<'a> Text<'a> {
 pub trait PromptMany {
     /// Calls prompt on a collection of [Text] instances and return their respective
     /// responses or the first error that appears.
-    fn prompt(self) -> Result<Vec<String>, Box<dyn Error>>;
+    fn prompt(self) -> InquireResult<Vec<String>>;
 }
 
 impl<'a, I> PromptMany for I
 where
     I: Iterator<Item = Text<'a>>,
 {
-    fn prompt(self) -> Result<Vec<String>, Box<dyn Error>> {
+    fn prompt(self) -> InquireResult<Vec<String>> {
         self.map(Text::prompt).collect()
     }
 }
@@ -140,7 +136,7 @@ struct TextPrompt<'a> {
     help_message: Option<&'a str>,
     content: String,
     formatter: StringFormatter,
-    validators: Vec<StringValidator>,
+    validators: Vec<StringValidator<'a>>,
     error: Option<String>,
     suggester: Option<Suggester>,
     suggested_options: Vec<String>,
@@ -209,19 +205,16 @@ impl<'a> TextPrompt<'a> {
         let mut dirty = false;
 
         match key {
+            Key::Tab => self.use_select_option(),
             Key::Backspace => {
                 let len = self.content[..].graphemes(true).count();
                 let new_len = len.saturating_sub(1);
                 self.content = self.content[..].graphemes(true).take(new_len).collect();
                 dirty = true;
             }
-            Key::Up => self.move_cursor_up(),
-            Key::Down => self.move_cursor_down(),
-            Key::Char('\x17') | Key::Char('\x18') => {
-                self.content.clear();
-                dirty = true;
-            }
-            Key::Char(c) => {
+            Key::Up(KeyModifiers::NONE) => self.move_cursor_up(),
+            Key::Down(KeyModifiers::NONE) => self.move_cursor_down(),
+            Key::Char(c, KeyModifiers::NONE) => {
                 self.content.push(c);
                 dirty = true;
             }
@@ -260,7 +253,7 @@ impl<'a> TextPrompt<'a> {
         Ok(self.content.clone())
     }
 
-    fn render(&mut self, renderer: &mut Renderer) -> Result<(), std::io::Error> {
+    fn render(&mut self, renderer: &mut Renderer) -> InquireResult<()> {
         let prompt = &self.message;
 
         renderer.reset_prompt()?;
@@ -294,7 +287,7 @@ impl<'a> TextPrompt<'a> {
         Ok(())
     }
 
-    fn prompt(mut self, renderer: &mut Renderer) -> Result<String, Box<dyn Error>> {
+    fn prompt(mut self, renderer: &mut Renderer) -> InquireResult<String> {
         let final_answer: String;
 
         loop {
@@ -303,9 +296,8 @@ impl<'a> TextPrompt<'a> {
             let key = renderer.read_key()?;
 
             match key {
-                Key::Ctrl('c') => bail!("Input interrupted by ctrl-c"),
-                Key::Char('\t') => self.use_select_option(),
-                Key::Char('\n') | Key::Char('\r') => match self.get_final_answer() {
+                Key::Cancel => return Err(InquireError::OperationCanceled),
+                Key::Submit => match self.get_final_answer() {
                     Ok(answer) => {
                         final_answer = answer;
                         break;
@@ -324,6 +316,7 @@ impl<'a> TextPrompt<'a> {
 
 #[cfg(test)]
 mod test {
+    use crossterm::event::{KeyCode, KeyEvent};
     use ntest::timeout;
 
     use crate::{renderer::Renderer, terminal::Terminal};
@@ -334,6 +327,12 @@ mod test {
         Text::new("Question?")
     }
 
+    macro_rules! text_to_events {
+        ($text:expr) => {{
+            $text.chars().map(|c| KeyCode::Char(c))
+        }};
+    }
+
     macro_rules! text_test {
         ($name:ident,$input:expr,$output:expr) => {
             text_test! {$name, $input, $output, default()}
@@ -343,10 +342,11 @@ mod test {
             #[test]
             #[timeout(100)]
             fn $name() {
-                let mut read: &[u8] = $input.as_bytes();
+                let read: Vec<KeyEvent> = $input.into_iter().map(KeyEvent::from).collect();
+                let mut read = read.iter();
 
                 let mut write: Vec<u8> = Vec::new();
-                let terminal = Terminal::new_with_io(&mut write, &mut read).unwrap();
+                let terminal = Terminal::new_with_io(&mut write, &mut read);
                 let mut renderer = Renderer::new(terminal).unwrap();
 
                 let ans = $prompt.prompt_with_renderer(&mut renderer).unwrap();
@@ -356,35 +356,77 @@ mod test {
         };
     }
 
-    text_test!(empty, "\n", "");
+    text_test!(empty, vec![KeyCode::Enter], "");
 
-    text_test!(single_letter, "b\n", "b");
+    text_test!(single_letter, vec![KeyCode::Char('b'), KeyCode::Enter], "b");
 
-    text_test!(letters_and_enter, "normal input\n", "normal input");
+    text_test!(
+        letters_and_enter,
+        text_to_events!("normal input\n"),
+        "normal input"
+    );
 
     text_test!(
         letters_and_enter_with_emoji,
-        "with emoji 🧘🏻‍♂️, 🌍, 🍞, 🚗, 📞\n",
+        text_to_events!("with emoji 🧘🏻‍♂️, 🌍, 🍞, 🚗, 📞\n"),
         "with emoji 🧘🏻‍♂️, 🌍, 🍞, 🚗, 📞"
     );
 
     text_test!(
         input_and_correction,
-        "anor\x7F\x7F\x7F\x7Fnormal input\n",
+        {
+            let mut events = vec![];
+            events.append(&mut text_to_events!("anor").collect());
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.append(&mut text_to_events!("normal input").collect());
+            events.push(KeyCode::Enter);
+            events
+        },
         "normal input"
     );
 
     text_test!(
         input_and_excessive_correction,
-        "anor\x7F\x7F\x7F\x7F\x7F\x7F\x7F\x7Fnormal input\n",
+        {
+            let mut events = vec![];
+            events.append(&mut text_to_events!("anor").collect());
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.append(&mut text_to_events!("normal input").collect());
+            events.push(KeyCode::Enter);
+            events
+        },
         "normal input"
     );
 
     text_test!(
         input_correction_after_validation,
-        "1234567890\n\x7F\x7F\x7F\x7F\x7F\nyes\n",
+        {
+            let mut events = vec![];
+            events.append(&mut text_to_events!("1234567890").collect());
+            events.push(KeyCode::Enter);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.push(KeyCode::Backspace);
+            events.append(&mut text_to_events!("yes").collect());
+            events.push(KeyCode::Enter);
+            events
+        },
         "12345yes",
-        Text::new("").with_validator(|ans| match ans.len() {
+        Text::new("").with_validator(&|ans| match ans.len() {
             len if len > 5 && len < 10 => Ok(()),
             _ => Err("Invalid".to_string()),
         })
